#-------------------------------------------------------------
# PD Report Generation subroutines
# 
# Called from the pdadmin.cgi menu.
#-------------------------------------------------------------

sub GenReport {
	my ($logfile,$reporttype) = @_;

	CheckBadPath($logfile);

	my $total=0;
	my $top=10;
	my ($detail,$description);

	my $site_url = $ENV{'SERVER_NAME'};
	my $midi_url = "$pdurl/" . basename($midi_dir);


	%description = ( 'date'		=> 'Date',
			 'type' 	=> 'Sending Method',
			 'design'	=> 'Postcard Design',
			 'music'	=> 'Midi Sound',
			 'image'	=> 'Postcard Image',
			 'object'	=> 'Postcard Object',
			 'remotehost'	=> 'Initiated From',
			 'sender'	=> 'Sender',
			 'recipient'	=> 'Recipient'
			);

	open (LOG,"$log_dir/$logfile") or Error("Can not open log: $log_dir/$logfile",$!);
	RHeader($reporttype);


	while (<LOG>) {
		if (m/\|/) {
			$total++;
			@field=split(/\|/);
			$date{$field[0]}++;
			$type{$field[2]}++;
			$design{$field[3]}++;
			$music{$field[4]}++ unless ($field[4] eq 'none');
			$object{$field[5]}++ if $field[5];
			$image{$field[6]}++ if $field[6];
			$remotehost{$field[7]}++;
			$sender{$field[9]}++;
			$recipient{$field[11]}++;;
	
			next if ($reporttype eq 'summary');

			$detail .= "<p><hr></p>\n";
			$detail .= qq(<table border="0">\n);
			$detail .= Row('Date',$field[0]);
			$detail .= Row('Time',$field[1]);
			$detail .= Row('Type',$field[2]);
			$detail .= Row('Design',$field[3]);

			if ($field[4] eq 'none') {
				$detail .= Row('Music',$field[4]);
			}
			else {
				$detail .= Row('Music',$field[4],"http://$site_url$midi_url/$field[4]");
			}

			$detail .= Row('Object',$field[5],"http://$site_url$field[5]") if $field[5];
			$detail .= Row('Image',$field[6],"http://$site_url$field[6]") if $field[6];
	
			$detail .= Row('Remote Host',$field[7]);
			$detail .= Row('Subject',$field[8]);
			$detail .= Row('Sender Email',$field[9]);
			$detail .= Row('Sender Name',$field[10]);
			$detail .= Row('Recipient Email',$field[11]);
			$detail .= Row('Recipient Name',$field[12]);
			$detail .= Row('Send Date',$field[13]);
			$detail .= Row('Field1',$field[14]);
			$detail .= Row('Field2',$field[15]);
			$detail .= Row('Field3',$field[16]);
			$detail .= Row('Field4',$field[17]);
			$detail .= Row('Field5',$field[18]);
			$detail .= Row('Field6',$field[19]);
			$detail .= Row('BCC',$field[20]);
			$detail .= Row('Receipt',$field[21]);
			$detail .= Row('Send Copy',$field[22]);
	
			$detail .= "</table>\n";
			$detail .= qq(<br>$field[23]\n);
		}
		else {
			$detail .= qq(<br>$_\n);
		}

	}

	close (LOG);

	print "<br><hr><br>\n";
	print "<h3><b>Total Entries Processed:</b> $total</h3>\n";

	my (@data,$counter);

	my @list=qw(date type design music object image remotehost sender recipient);
	foreach my $column (@list) {
		$counter=0;

		@data=();

		while (my ($key,$value) = each %$column) {
                	push(@data,"$key|$value");
		}


		next unless @data;

		#
		# Sort on the second field
		#
		my @sortdata = map { $_->[0] }
		    	sort { $b->[2] <=> $a->[2] }
		    	map { [ $_,(split /\|/) ] }
		    	@data;

		print qq(<table border="2" cellspacing="2" cellpadding="2" width="80%">\n);
		print qq(<tr class="theader">\n);
 		print qq(<td width="60%"><div class="title">$description{$column}</div></td>\n);
 		print qq(<td width="10%"><div class="title">Entries</div></td>\n);
 		print qq(<td width="30%"><div class="title">Percent</div></td>\n</tr>\n);

		foreach (@sortdata) {
			last if $counter > $top;

			my ($entry,$count)=split(/\|/);
			my $percent=sprintf("%3.2f",($count / $total) *100);

			print qq(<tr class="tbody"><td>$entry</td><td align="right">$count</td>\n);
			print qq(<td><hr noshade class="bar" align="left" size="10" width="$percent%"> $percent%</td></tr>\n);
			$counter++;
		}

		print "</table><p>\n"; 
	}

	if ($reporttype eq 'detailed') {
		print "<p><hr></p>\n";
		print "<h3>Log Details</h3>\n";
		print $detail;
	}

	RFooter();
}

#-----------------------------------------------------------------------------
sub RHeader {
	my $reporttype=shift;

	open(STYLE,"$style_dir/usagereport.css") or Error("Can not open $style_dir/usagereport.css",$!);
	my @style=<STYLE>;
	close(STYLE);

	print <<HTML;
<html>
<head>
<meta http-equiv="pragma" content="no-cache">
<title>Postcard Direct Log Report</title>
@style
</head>

<body>

<h1>Postcard Direct Log Report</h1>

HTML

	if ($reporttype eq 'detailed') {
		print "<b>Report Type:</b> Summary and Detail<br>\n";
	}
	else {
		print "<b>Report Type:</b> Summary Only<br>\n";
	}

	print "<b>Report Generated at:</b> " . localtime() . "<br>\n";
}


#-----------------------------------------------------------------------------
sub Row {
	my ($name,$value,$url) = @_;

	if ($url) {
		return qq(<tr class="field"><td><b>$name:</b></td><td><a href="$url">$value</a></td>\n);
	}
	else {
		return qq(<tr class="field"><td><b>$name:</b></td><td>$value</td>\n);
	}
}

#-----------------------------------------------------------------------------
sub RFooter {
	print <<HTML;
<hr>
<p>
<form>
<input type="button" value="Close Window" onClick="window.close()">
<input type="button" value="Generate another report" onClick="history.go(-1);return true">
</form>
</p>

</body>
</html>
HTML
}

#-----------------------------------------------------------------------------

1;
